module Resources

  class Tube < BaseResource
    include TubePrompter
    include ToTubePrompter

    def list
      result = get("beanstalk/tubes")
      puts "\nBeanstalk tubes"
      table = Terminal::Table.new headings: [ 'Tube Name' ] do | t |
        result.each do | tube_name |
          t << [ tube_name ]
        end
      end
      puts table
      puts
    end

    def show
      tube = tube_name
      result = get("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}")
      puts "\nStatistics for tube: #{tube}"
      table = Terminal::Table.new headings: [ 'Key', 'Value' ] do | t |
        result.each_pair do | key, value |
          t << [ key, value ]
        end
      end
      puts table
      puts
    end

    def ready
      tube = tube_name
      result = get("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/ready")
      if result.has_key?('error')
        puts
        puts result['error']
      else
        puts "\nJobs in the Ready state:"
        table = Terminal::Table.new headings: [ 'Id', 'Body' ] do | t |
          t << [ result.keys.first, result.values.first ]
        end
        puts table
      end
      puts
    end

    def delayed
      tube = tube_name
      result = get("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/delayed")
      if result.has_key?('error')
        puts
        puts result['error']
      else
        puts "\nJobs in the Delayed state:"
        table = Terminal::Table.new headings: [ 'Id', 'Body' ] do | t |
          t << [ result.keys.first, result.values.first ]
        end
        puts table
      end
      puts
    end

    def buried
      tube = tube_name
      result = get("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/buried")
      if result.has_key?('error')
        puts
        puts result['error']
      else
        puts "\nJobs in the Buried state:"
        table = Terminal::Table.new headings: [ 'Id', 'Body' ] do | t |
          t << [ result.keys.first, result.values.first ]
        end
        puts table
      end
      puts
    end

    def delete_one_job
      tube = tube_name
      result = delete("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/one_job")
      puts "\nJob deleted"
      puts result['eat_job']
      puts
    end

    def delete_all_jobs
      tube = tube_name
      puts
      state = prompter.choose do | menu |
        menu.prompt = "Select the state of the jobs to be deleted: "
        menu.choice('ready', 'All jobs in the ready state') { 'ready' }
        menu.choice('buried', 'All jobs in the buried state') { 'buried' }
      end
      result = delete("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/all_jobs?state=#{state}")
      puts "\nAll jobs deleted"
      puts result['eat_job']
      puts
    end

    def move
      tube = tube_name
      to_tube = to_tube_name
      result = delete("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/move/#{Base64.urlsafe_encode64(to_tube)}")
      puts "\nAll jobs moved from '#{tube}' to '#{to_tube}':"
      puts result['eat_job']
      puts
    end

    def kill_worker
      tube = tube_name
      result = delete("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/worker")
      puts "\nWorker killed:"
      puts result['kill_worker']
      puts
    end

    def kick
      tube = tube_name
      num_jobs = prompter.ask("Number of jobs to kick: ", Integer) { |q| q.default = 1 }
      result = get("beanstalk/tubes/#{Base64.urlsafe_encode64(tube_name)}/kick/#{num_jobs}")
      puts "\nKick #{num_jobs} in tube '#{tube}':"
      puts result['kick']
      puts
    end

  end

end
